/* L-Dic(Linux Dictionary)
 * Copyright (C) 2002 Baek SeungSoon.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <malloc.h>
#include <string.h>
#include <getopt.h>

#include "ldic.h"
#include "wlist.h"

#define LDIC_DIR "/usr/share/ldic"
#define LDIC_VER "0.0.1"

WList *wlist;

/* readList:   о Ʈ ߰
 */
static int
readList(char *listName)
{
	FILE *fp;

	fp = fopen(listName, "r");

	if (fp == NULL)
		return 1;

	for (;;) {
		long w_offset;
		char buf[BUFSIZ+1];
		
		fscanf(fp, "%ld,%s", &w_offset, buf);
		if (feof(fp))
			break;

		wlist = addfront(wlist, newitem(w_offset, buf));
	}
	
	fclose (fp);

	return 0;
}

/* fexist:  ϴ ˻
 */
static int
fexist(char *filename)
{
	FILE *fp;

	fp = fopen(filename, "r");
	if (fp == NULL) {
		return 0;
	} else {
		fclose(fp);
		return 1;
	}
}

/* printPhon: ܾ ȣ 
 */
static void
printPhon(char *s)
{
	char **phonlist;

	phonlist = tagList(s, "img src");
	printf("[_phonetic symbol_]\n");
	PrintXpm(phonlist, LDIC_DIR);
	printf("[_done_]\n");
}

/* printMean: ܾ  
 */
static void
printMean(char *s)
{
	char *mean;

	mean = tagRemove(s);
	printf("[_mean_]\n");
	printf("%s\n", mean);
	printf("[_done_]\n");
}

/* getErrMsg: err شϴ ޼ ȯ
 */
static char
*getErrMsg(char *err)
{
	static char retval[BUFSIZ+1];
	
	char *errList[] = {
		"list", " ã  ϴ.",
		"dic",  " ã  ϴ.",
		"word", "ܾ ã  ϴ.",
		"cmd",  "ӵ  ƴմϴ.",
		NULL
	};

	int i;

	for (i = 0; errList[i] != NULL; i += 2) {
		if (strcmp(err, errList[i]) == '\0') {
			strcpy(retval, errList[i+1]);
			break;
		}
	}

	if (errList[i] == NULL) {
		strcpy(retval, "    ߻߽ϴ.");
	}

	return retval;
}

/* usage: α׷  
 */
static void
usage(const char *str)
{
	char *s;
	
	if ((s = strrchr(str, '/')) != NULL) {
		++s;
	} else {
		s = strdup(str);
	}

	printf(": %s [ɼ]... [ܾ]\n", s);
	printf("\n");
	printf("  -q WORD,\t˻  \n");
	printf("  -p,     \tȣ \n");
	printf("  -d FILE,\t \n");
	printf("  -l,     \tܾ  \n");
	printf("  -o,     \ttag ž \n");
	printf("  -h,     \t   \n");
	printf("  -v,     \tα׷    \n");
	printf("\n");
	printf("<tadboy@ldic.kldp.net>    ּ\n");
	
	exit(0);
}

/* version: α׷  
 */
static void
version(void)
{
	printf("L-Dic(Linux Dictionary) Version %s\n", LDIC_VER);
	printf("¼  ϴ.\n\n");
	printf("Copyright (C) 2002 Free Software Foundation, Inc.\n");
	printf(" α׷  ƮԴϴ. "
		"  ҽ Ͻʽÿ. "
		"ǰ̳ Ư   ռ Ͽ, "
		"   ʽϴ.\n");
}

/* main: α׷ ϰ, ɼ ó
 */
int
main(int argc, char *argv[])
{
	char c;
	
	int quickOpt = 0;	/* ˻ ɼ */
	int phonOpt = 0;	/* ȣ  */
	int dbOpt = 0;		/*   */
	int listOpt = 0;	/* ܾ   */
	int origOpt = 0;	/* tag ž  */
	
	char dicName[BUFSIZ+1];
	char listName[BUFSIZ+1];
	char word[BUFSIZ+1];

	char *s;
	long w_offset;

	while ((c =  getopt(argc, argv,	"q:pd:loh?v")) != -1) {
		switch (c) {
			case 'q':
				quickOpt = 1;
				strcpy(word, optarg);
				break;
			case 'p':
				phonOpt = 1;
				break;
			case 'd':
				dbOpt = 1;
				strcpy(dicName, optarg);
				break;
			case 'l':
				listOpt = 1;
				break;
			case 'o':
				origOpt = 1;
				break;
			case 'h':
			case '?':
				usage(argv[0]);
				break;
			case 'v':
				version();
				exit(0);
			default:
				break;
		}
	}

	if (optind < argc)
		quickOpt = 1;

	for (; optind < argc; ++optind) {
		strcat(word, argv[optind]);
	}
	
	if (dbOpt == 0) {
		sprintf(dicName, "%s/dictionary/ldic.dic", LDIC_DIR);
	}

	if (fexist(dicName) == 0) {
		fprintf(stderr, "Error: %s\n", getErrMsg("dic"));
		exit(1);
	}

	strcpy(listName, dicName);
	s = strrchr(listName, '.');
	if (s != NULL)
		*s = '\0';
	
	strcat(listName, ".list");
	
	if (fexist(listName) == 0) {
		fprintf(stderr, "Error: %s\n", getErrMsg("list"));
		listOpt = 1;
	}

	if (listOpt == 1) {
		write(1, "Words List Making...\t\t", 23);
		if (MakeList(dicName, listName) != 0) {
			fprintf(stderr, "[Error]\n");
			fprintf(stderr, " ų, 뷮 մϴ.\n");
			exit(1);
		}
		printf("[Done]\n");
	}

	if (quickOpt == 1) {
		w_offset = FileSearchWord(word, listName);
		if (w_offset == -1) {
			fprintf(stderr, "Error: %s\n", getErrMsg("word"));
			exit(1);
		}
		
		s = dbSearchMean(w_offset, dicName);
		if (s == NULL) {
			fprintf(stderr, "Error: %s\n", getErrMsg("word"));
			exit(1);
		}

		if (phonOpt == 1) {
			printPhon(s);
		} else if (origOpt == 1) {
			printf("[_orig mean_]\n%s\n[_done_]\n", s);
		} else {
			printMean(s);
		}
	} else {
		write(1, "Words List Loding...\t\t", 23);
		if(readList(listName) != 0) {
			fprintf(stderr, "[Error]\n");
			exit(1);
		}
		printf("[Done]\n");

		for (;;) {
			write(1, "Search Word ('/exit'): ", 23);
			
			fgets(word, sizeof(word), stdin);
			word[strlen(word)-1] = '\0';

			if (word[0] == '/') {
				if(strcmp(word, "/exit") == '\0') {
					break;
				} else {
					printf("Error: %s\n", getErrMsg("cmd"));
				}
				continue;
			}

			w_offset = ListSearchWord(wlist, word);

			if (w_offset == -1) {
				printf("Error: %s\n", getErrMsg("word"));
				continue;
			} else {
				s = dbSearchMean(w_offset, dicName);
				
				if (phonOpt == 1) {
					printPhon(s);
				} else if (origOpt == 1) {
					printf("[_orig mean_]\n%s\n[_done_]\n", s);
				} else {
					printMean(s);
				}
			}
		}

		freeall(wlist);
	}

	return 0;
}
